/*
  Примеры для книги Белова Александра
  Программа 17.2.   Глава 17
  Кодовый замок

 Схема:
 * Десять кнопок подключаются к пинам с 0 по 9,
 * Тумблер выбора режима подключается между пином 10 и общим проводом
 * Внешний ключ управления электромагнитом подключается через резистор 2 кОм к пину 13
 * Открывание двери так же будет индицироваться светодиодом, установленном на плате
 */

#include <EEPROM.h>

#define klfree 0x3FF    // Код состояния при полностью отпущеных кнопках
#define kzad 3000       // Код задержки при сканировании
#define kandr 30        // Константа антидребезга
#define bsize 30        // Размер буфера для хранения кода
#define klen bsize*2+1  // Номер ячейки EEPROM для хранения длинны кода
#define modePin 11      // Пин переключателя режима
#define relayPin 13     // Еомер пина для подключения исполнительного механизма замка

unsigned char ii;       // Указатель массива
unsigned char i;        // Вспомогательный указатель
unsigned int codS;      // Старый код

unsigned int bufr[bsize+1];   // Буфер в ОЗУ для хранения кода



void setup() {
  pinMode(relayPin, OUTPUT);
  // инициализируем пины кнопок на ввод:
  for (int ButtonPin=0; ButtonPin<=9; ButtonPin++) {
    pinMode(ButtonPin, INPUT_PULLUP);
  }
  pinMode(modePin, INPUT_PULLUP);    // Инициализация входа переключателя режимов
}


void loop() {
     unsigned long itime;
     unsigned int  CodC;
     
m1:  while (incod() != klfree); // Ожидание отпускания кнопок
     while (incod() == klfree); // Ожидание нажатия кнопок
     ii=0;
m2:  delay(50);                 // Задержка 50 мс
     codS=incod();              // Ввод кода и запись, как старого
     bufr[ii++]=codS;           // Запись очередного кода в буфер
     if (ii>=bsize) goto m4;    // Проверка конца буфера
        
     itime = millis();                     // Сканируем состояние кнопок в течении 1 секунды
m3:  if (incod() != codS) goto m2;         // Проверка не изменилось ли состояние
     //Serial.println(incod(),BIN);
     if (millis()-itime<1000) goto m3;     // Проверка окончания контрольного промежутка времени
        
m4:  if (digitalRead(modePin)==HIGH) goto comp;       // Проверка переключателя режимов

//------------------------------ Запись кода в EEPROM
     EEPROM.write(klen,ii);       // Запись длины кода
     // Запись всех байтов кода
     for (i=0; i<ii; i++)  {
        EEPROM.write(i*2,highByte(bufr[i]));
        EEPROM.write(i*2+1,lowByte(bufr[i]));
     }
     goto zamok;

//------------------------------ Проверка кода
comp:   if (EEPROM.read(klen)!=ii) goto m1;                 // Проверка длины кода
        for (i=0; i<ii; i++)  {
          CodC = EEPROM.read(i*2)*256 + EEPROM.read(i*2+1);
          if (CodC!=bufr[i]) goto m1;                       // Проверка самого кода
        }

//------------------------------ Открывание замка
zamok:  digitalWrite(relayPin,HIGH);        // Открываем замок
        delay(1000);                        // Задержка на 1 сек
        digitalWrite(relayPin,LOW);         // Закрываем замок

}


// Функция опроса клавиатуры и антидребезга
unsigned int incod (void)  {
   unsigned int cod0=0;  // Локальные переменные
   unsigned int cod1;
   byte k;
   
   // Цикл антидребезга
   for (k=0; k<kandr; k++) {
     cod1=0;
     for (int ButtonPin=0; ButtonPin<=9; ButtonPin++) {
       cod1=(cod1<<1)+ digitalRead(ButtonPin);
     }
     if (cod0!=cod1) {          // Сравниваем со старым кодом
       k=0;                    // Если не равны, сбрасываем счетчик
       cod0=cod1;              // И присваиваем новое значение старому коду
      }
   }
   return cod1;
}

